@@ -23,6 +23,8 @@ import subprocess
 import tarfile
 import pipes
 import platform
+import zipfile
+import shutil
 
 try:  # pragma: no cover (py2 only)
     from ConfigParser import SafeConfigParser as ConfigParser
@@ -37,7 +39,7 @@ except ImportError:  # pragma: no cover (py3 only)
 
 from pkg_resources import parse_version
 
-nodeenv_version = '1.0.0'
+nodeenv_version = '1.1.1'
 
 join = os.path.join
 abspath = os.path.abspath
@@ -84,7 +86,7 @@ class Config(object):
     # Defaults
     node = 'latest'
     npm = 'latest'
-    with_npm = False
+    with_npm = True if is_WIN else False
     jobs = '2'
     without_ssl = False
     debug = False
@@ -196,6 +198,8 @@ def create_logger():
     # add ch to logger
     logger.addHandler(ch)
     return logger
+
+
 logger = create_logger()
 
 
@@ -222,15 +226,42 @@ def parse_args(check=True):
         action='store_true', dest='io', default=False,
         help='Use iojs instead of nodejs.')
 
-    parser.add_option(
-        '-j', '--jobs', dest='jobs', default=Config.jobs,
-        help='Sets number of parallel commands at node.js compilation. '
-        'The default is 2 jobs.')
+    if not is_WIN:
+        parser.add_option(
+            '-j', '--jobs', dest='jobs', default=Config.jobs,
+            help='Sets number of parallel commands at node.js compilation. '
+            'The default is 2 jobs.')
 
-    parser.add_option(
-        '--load-average', dest='load_average',
-        help='Sets maximum load average for executing parallel commands '
-             'at node.js compilation.')
+        parser.add_option(
+            '--load-average', dest='load_average',
+            help='Sets maximum load average for executing parallel commands '
+            'at node.js compilation.')
+
+        parser.add_option(
+            '--without-ssl', dest='without_ssl',
+            action='store_true', default=Config.without_ssl,
+            help='Build node.js without SSL support')
+
+        parser.add_option(
+            '--debug', dest='debug',
+            action='store_true', default=Config.debug,
+            help='Build debug variant of the node.js')
+
+        parser.add_option(
+            '--profile', dest='profile',
+            action='store_true', default=Config.profile,
+            help='Enable profiling for node.js')
+
+        parser.add_option(
+            '--make', '-m', dest='make_path',
+            metavar='MAKE_PATH',
+            help='Path to make command',
+            default=Config.make)
+
+        parser.add_option(
+            '--source', dest='prebuilt',
+            action='store_false', default=Config.prebuilt,
+            help='Install node.js from the source')
 
     parser.add_option(
         '-v', '--verbose',
@@ -266,27 +297,12 @@ def parse_args(check=True):
         action='store_true', default=False,
         help='Install npm packages from file without node')
 
-    parser.add_option(
-        '--without-ssl', dest='without_ssl',
-        action='store_true', default=Config.without_ssl,
-        help='Build node.js without SSL support')
-
-    parser.add_option(
-        '--debug', dest='debug',
-        action='store_true', default=Config.debug,
-        help='Build debug variant of the node.js')
-
-    parser.add_option(
-        '--profile', dest='profile',
-        action='store_true', default=Config.profile,
-        help='Enable profiling for node.js')
-
     parser.add_option(
         '--with-npm', dest='with_npm',
         action='store_true', default=Config.with_npm,
         help='Build without installing npm into the new virtual environment. '
         'Required for node.js < 0.6.3. By default, the npm included with '
-        'node.js is used.')
+        'node.js is used. Under Windows, this defaults to true.')
 
     parser.add_option(
         '--npm', dest='npm',
@@ -316,22 +332,11 @@ def parse_args(check=True):
         action='store_true', default=False,
         help='Force installation in a pre-existing directory')
 
-    parser.add_option(
-        '--make', '-m', dest='make_path',
-        metavar='MAKE_PATH',
-        help='Path to make command',
-        default=Config.make)
-
     parser.add_option(
         '--prebuilt', dest='prebuilt',
         action='store_true', default=Config.prebuilt,
         help='Install node.js from prebuilt package (default)')
 
-    parser.add_option(
-        '--source', dest='prebuilt',
-        action='store_false', default=Config.prebuilt,
-        help='Install node.js from the source')
-
     options, args = parser.parse_args()
     if options.config_file is None:
         options.config_file = ["./setup.cfg", "~/.nodeenvrc"]
@@ -378,7 +383,7 @@ def writefile(dest, content, overwrite=True, append=False):
     mode_0755 = (stat.S_IRWXU | stat.S_IXGRP |
                  stat.S_IRGRP | stat.S_IROTH | stat.S_IXOTH)
     content = to_utf8(content)
-    if is_PY3:
+    if is_PY3 and type(content) != bytes:
         content = bytes(content, 'utf-8')
     if not os.path.exists(dest):
         logger.debug(' * Writing %s ... ', dest, extra=dict(continued=True))
@@ -402,9 +407,11 @@ def writefile(dest, content, overwrite=True, append=False):
         if append:
             logger.info(' * Appending data to %s', dest)
             with open(dest, 'ab') as f:
-                f.write(DISABLE_PROMPT.encode('utf-8'))
+                if not is_WIN:
+                    f.write(DISABLE_PROMPT.encode('utf-8'))
                 f.write(content)
-                f.write(ENABLE_PROMPT.encode('utf-8'))
+                if not is_WIN:
+                    f.write(ENABLE_PROMPT.encode('utf-8'))
             return
 
         logger.info(' * Overwriting %s with new content', dest)
@@ -524,18 +531,22 @@ def download_node_src(node_url, src_dir, opt, prefix):
     Download source code
     """
     logger.info('.', extra=dict(continued=True))
-    tar_contents = io.BytesIO(urlopen(node_url).read())
+    dl_contents = io.BytesIO(urlopen(node_url).read())
     logger.info('.', extra=dict(continued=True))
-    with tarfile_open(fileobj=tar_contents) as tarfile_obj:
-        member_list = tarfile_obj.getmembers()
-        extract_list = []
-        for member in member_list:
-            node_ver = opt.node.replace('.', '\.')
-            regex_string = "%s-v%s[^/]*/(README\.md|CHANGELOG\.md|LICENSE)" \
-                % (prefix, node_ver)
-            if re.match(regex_string, member.name) is None:
-                extract_list.append(member)
-        tarfile_obj.extractall(src_dir, extract_list)
+
+    if is_WIN:
+        writefile(join(src_dir, 'node.exe'), dl_contents.read())
+    else:
+        with tarfile_open(fileobj=dl_contents) as tarfile_obj:
+            member_list = tarfile_obj.getmembers()
+            extract_list = []
+            for member in member_list:
+                node_ver = opt.node.replace('.', '\.')
+                rexp_string = "%s-v%s[^/]*/(README\.md|CHANGELOG\.md|LICENSE)"\
+                    % (prefix, node_ver)
+                if re.match(rexp_string, member.name) is None:
+                    extract_list.append(member)
+            tarfile_obj.extractall(src_dir, extract_list)
 
 
 def urlopen(url):
@@ -548,13 +559,20 @@ def urlopen(url):
 # Virtual environment functions
 
 
-def copy_node_from_prebuilt(env_dir, src_dir):
+def copy_node_from_prebuilt(env_dir, src_dir, node_version):
     """
     Copy prebuilt binaries into environment
     """
     logger.info('.', extra=dict(continued=True))
     prefix = get_binary_prefix()
-    callit(['cp', '-a', src_dir + '/%s-v*/*' % prefix, env_dir], True, env_dir)
+    if is_WIN:
+        src_exe = join(src_dir, 'node.exe')
+        dst_exe = join(env_dir, 'Scripts', 'node.exe')
+        mkdir(join(env_dir, 'Scripts'))
+        callit(['copy', '/Y', '/L', src_exe, dst_exe], False, True)
+    else:
+        src_folder = src_dir + to_utf8('/%s-v%s*/*' % (prefix, node_version))
+        callit(['cp', '-a', src_folder, env_dir], True, env_dir)
     logger.info('.', extra=dict(continued=True))
 
 
@@ -642,7 +660,7 @@ def install_node(env_dir, src_dir, opt):
     logger.info('.', extra=dict(continued=True))
 
     if opt.prebuilt:
-        copy_node_from_prebuilt(env_dir, src_dir)
+        copy_node_from_prebuilt(env_dir, src_dir, opt.node)
     else:
         build_node_from_src(env_dir, src_dir, node_src_dir, opt)
 
@@ -680,6 +698,39 @@ def install_npm(env_dir, src_dir, opt):
     logger.info('done.')
 
 
+def install_npm_win(env_dir, src_dir, opt):
+    """
+    Download source code for npm, unpack it
+    and install it in virtual environment.
+    """
+    logger.info(' * Install npm.js (%s) ... ' % opt.npm,
+                extra=dict(continued=True))
+    npm_url = 'https://github.com/npm/npm/archive/%s.zip' % opt.npm
+    npm_contents = io.BytesIO(urlopen(npm_url).read())
+
+    bin_path = join(env_dir, 'Scripts')
+    node_modules_path = join(bin_path, 'node_modules', 'npm')
+
+    if os.path.exists(node_modules_path):
+        shutil.rmtree(node_modules_path)
+
+    if os.path.exists(join(bin_path, 'npm.cmd')):
+        os.remove(join(bin_path, 'npm.cmd'))
+
+    if os.path.exists(join(bin_path, 'npm-cli.js')):
+        os.remove(join(bin_path, 'npm-cli.js'))
+
+    with zipfile.ZipFile(npm_contents, 'r') as zipf:
+        zipf.extractall(src_dir)
+
+    npm_ver = 'npm-%s' % opt.npm
+    shutil.copytree(join(src_dir, npm_ver), node_modules_path)
+    shutil.copy(join(src_dir, npm_ver, 'bin', 'npm.cmd'),
+                join(bin_path, 'npm.cmd'))
+    shutil.copy(join(src_dir, npm_ver, 'bin', 'npm-cli.js'),
+                join(bin_path, 'npm-cli.js'))
+
+
 def install_packages(env_dir, opt):
     """
     Install node.js packages via npm
@@ -711,15 +762,27 @@ def install_activate(env_dir, opt):
     """
     Install virtual environment activation script
     """
-    files = {'activate': ACTIVATE_SH, 'shim': SHIM}
+    if is_WIN:
+        files = {
+            'activate.bat': ACTIVATE_BAT,
+            "deactivate.bat": DEACTIVATE_BAT,
+            "Activate.ps1": ACTIVATE_PS1
+        }
+        bin_dir = join(env_dir, 'Scripts')
+        shim_node = join(bin_dir, "node.exe")
+        shim_nodejs = join(bin_dir, "nodejs.exe")
+    else:
+        files = {'activate': ACTIVATE_SH, 'shim': SHIM}
+        bin_dir = join(env_dir, 'bin')
+        shim_node = join(bin_dir, "node")
+        shim_nodejs = join(bin_dir, "nodejs")
+
     if opt.node == "system":
         files["node"] = SHIM
-    bin_dir = join(env_dir, 'bin')
+
     mod_dir = join('lib', 'node_modules')
     prompt = opt.prompt or '(%s)' % os.path.basename(os.path.abspath(env_dir))
 
-    shim_node = join(bin_dir, "node")
-    shim_nodejs = join(bin_dir, "nodejs")
     if opt.node == "system":
         env = os.environ.copy()
         env.update({'PATH': remove_env_bin_from_path(env['PATH'], bin_dir)})
@@ -750,7 +813,19 @@ def install_activate(env_dir, opt):
         writefile(file_path, content, append=need_append)
 
     if not os.path.exists(shim_nodejs):
-        os.symlink("node", shim_nodejs)
+        if is_WIN:
+            try:
+                callit(['mklink', shim_nodejs, 'node.exe'], True, True)
+            except OSError:
+                logger.error('Error: Failed to create nodejs.exe link')
+        else:
+            os.symlink("node", shim_nodejs)
+
+
+def set_predeactivate_hook(env_dir):
+    if not is_WIN:
+        with open(join(env_dir, 'bin', 'predeactivate'), 'a') as hook:
+            hook.write(PREDEACTIVATE_SH)
 
 
 def create_environment(env_dir, opt):
@@ -775,12 +850,15 @@ def create_environment(env_dir, opt):
     # for install
     install_activate(env_dir, opt)
     if node_version_from_opt(opt) < parse_version("0.6.3") or opt.with_npm:
-        install_npm(env_dir, src_dir, opt)
+        instfunc = install_npm_win if is_WIN else install_npm
+        instfunc(env_dir, src_dir, opt)
     if opt.requirements:
         install_packages(env_dir, opt)
+    if opt.python_virtualenv:
+        set_predeactivate_hook(env_dir)
     # Cleanup
     if opt.clean_src:
-        callit(['rm -rf', pipes.quote(src_dir)], opt.verbose, True, env_dir)
+        shutil.rmtree(src_dir)
 
 
 class GetsAHrefs(HTMLParser):
@@ -793,6 +871,7 @@ class GetsAHrefs(HTMLParser):
         if tag == 'a':
             self.hrefs.append(dict(attrs).get('href', ''))
 
+
 VERSION_RE = re.compile('\d+\.\d+\.\d+')
 
 
@@ -910,6 +989,10 @@ def main():
 
     opt, args = parse_args()
 
+    if opt.node.lower() == 'system' and is_WIN:
+        logger.error('Installing system node.js on win32 is not supported!')
+        exit(1)
+
     if opt.io:
         global src_domain
         src_domain = "iojs.org"
@@ -948,6 +1031,99 @@ export npm_config_prefix=__NODE_VIRTUAL_ENV__
 exec __SHIM_NODE__ "$@"
 """
 
+ACTIVATE_BAT = """\
+@echo off
+set "NODE_VIRTUAL_ENV=__NODE_VIRTUAL_ENV__"
+if not defined PROMPT (
+    set "PROMPT=$P$G"
+)
+if defined _OLD_VIRTUAL_PROMPT (
+    set "PROMPT=%_OLD_VIRTUAL_PROMPT%"
+)
+if defined _OLD_VIRTUAL_NODE_PATH (
+    set "NODE_PATH=%_OLD_VIRTUAL_NODE_PATH%"
+)
+set "_OLD_VIRTUAL_PROMPT=%PROMPT%"
+set "PROMPT=__NODE_VIRTUAL_PROMPT__ %PROMPT%"
+if defined NODE_PATH (
+    set "_OLD_VIRTUAL_NODE_PATH=%NODE_PATH%"
+    set NODE_PATH=
+)
+if defined _OLD_VIRTUAL_PATH (
+    set "PATH=%_OLD_VIRTUAL_PATH%"
+) else (
+    set "_OLD_VIRTUAL_PATH=%PATH%"
+)
+set "PATH=%NODE_VIRTUAL_ENV%\Scripts;%PATH%"
+:END
+
+"""
+
+DEACTIVATE_BAT = """\
+@echo off
+if defined _OLD_VIRTUAL_PROMPT (
+    set "PROMPT=%_OLD_VIRTUAL_PROMPT%"
+)
+set _OLD_VIRTUAL_PROMPT=
+if defined _OLD_VIRTUAL_NODE_PATH (
+    set "NODE_PATH=%_OLD_VIRTUAL_NODE_PATH%"
+    set _OLD_VIRTUAL_NODE_PATH=
+)
+if defined _OLD_VIRTUAL_PATH (
+    set "PATH=%_OLD_VIRTUAL_PATH%"
+)
+set _OLD_VIRTUAL_PATH=
+set NODE_VIRTUAL_ENV=
+:END
+"""
+
+ACTIVATE_PS1 = """\
+function global:deactivate ([switch]$NonDestructive) {
+    # Revert to original values
+    if (Test-Path function:_OLD_VIRTUAL_PROMPT) {
+        copy-item function:_OLD_VIRTUAL_PROMPT function:prompt
+        remove-item function:_OLD_VIRTUAL_PROMPT
+    }
+    if (Test-Path env:_OLD_VIRTUAL_NODE_PATH) {
+        copy-item env:_OLD_VIRTUAL_NODE_PATH env:NODE_PATH
+        remove-item env:_OLD_VIRTUAL_NODE_PATH
+    }
+    if (Test-Path env:_OLD_VIRTUAL_PATH) {
+        copy-item env:_OLD_VIRTUAL_PATH env:PATH
+        remove-item env:_OLD_VIRTUAL_PATH
+    }
+    if (Test-Path env:NODE_VIRTUAL_ENV) {
+        remove-item env:NODE_VIRTUAL_ENV
+    }
+    if (!$NonDestructive) {
+        # Self destruct!
+        remove-item function:deactivate
+    }
+}
+
+deactivate -nondestructive
+$env:NODE_VIRTUAL_ENV="__NODE_VIRTUAL_ENV__"
+
+# Set the prompt to include the env name
+# Make sure _OLD_VIRTUAL_PROMPT is global
+function global:_OLD_VIRTUAL_PROMPT {""}
+copy-item function:prompt function:_OLD_VIRTUAL_PROMPT
+function global:prompt {
+    Write-Host -NoNewline -ForegroundColor Green '__NODE_VIRTUAL_PROMPT__ '
+    _OLD_VIRTUAL_PROMPT
+}
+
+# Clear NODE_PATH
+if (Test-Path env:NODE_PATH) {
+    copy-item env:NODE_PATH env:_OLD_VIRTUAL_NODE_PATH
+    remove-item env:NODE_PATH
+}
+
+# Add the venv to the PATH
+copy-item env:PATH env:_OLD_VIRTUAL_PATH
+$env:PATH = "$env:NODE_VIRTUAL_ENV\Scripts;$env:PATH"
+"""
+
 ACTIVATE_SH = """
 
 # This file must be used with "source bin/activate" *from bash*
@@ -1075,5 +1251,9 @@ if [ -n "$BASH" -o -n "$ZSH_VERSION" ] ; then
 fi
 """
 
+PREDEACTIVATE_SH = """
+if type -p deactivate_node > /dev/null; then deactivate_node;fi
+"""
+
 if __name__ == '__main__':
     main()
